@@ -3,9 +3,6 @@
 # Univention Management Console
 #  Univention Directory Manager Module
 #
-# Like what you see? Join us!
-# https://www.univention.com/about-us/careers/vacancies/
-#
 # SPDX-FileCopyrightText: 2019-2025 Univention GmbH
 # SPDX-License-Identifier: AGPL-3.0-only
 
@@ -32,11 +29,10 @@ import tornado.web
 from setproctitle import getproctitle, setproctitle
 from tornado.netutil import bind_sockets, bind_unix_socket
 
-import univention.debug as ud
 import univention.lib.i18n
 import univention.logging
 from univention.admin.rest.shared_memory import shared_memory
-from univention.admin.rest.utils import RE_UUID, init_request_id_logging
+from univention.admin.rest.utils import RE_UUID, init_request_context_logging
 from univention.config_registry import ucr
 
 
@@ -46,8 +42,8 @@ except ImportError:
     _exit_function = None
 
 proctitle = getproctitle()
-request_id_context = contextvars.ContextVar("request_id")
-logger = logging.getLogger('ADMIN')
+request_context = contextvars.ContextVar("request_context")
+log = univention.logging.Structured(logging.getLogger('ADMIN'))
 
 
 class Gateway(tornado.web.RequestHandler):
@@ -70,7 +66,7 @@ class Gateway(tornado.web.RequestHandler):
     def prepare(self):
         request_id = self.request.headers.setdefault('X-Request-Id', str(uuid.uuid4()))
         self.request.x_request_id = RE_UUID.sub('', request_id)[:36]
-        request_id_context.set(self.request.x_request_id)
+        request_context.set({"request_id": self.request.x_request_id})
 
     @tornado.gen.coroutine
     def get(self):
@@ -96,7 +92,7 @@ class Gateway(tornado.web.RequestHandler):
         try:
             response = yield client.fetch(request, raise_error=True)
         except tornado.curl_httpclient.CurlError as exc:
-            ud.debug(ud.MAIN, ud.WARN, f'Reaching service failed: {exc}')
+            log.warning('Reaching service failed', error=exc)
             # happens during starting the service and subprocesses when the UNIX sockets aren't available yet
             self.set_status(503)
             self.add_header('Retry-After', '3')  # Tell clients, we are ready in 3 seconds
@@ -157,21 +153,18 @@ class Gateway(tornado.web.RequestHandler):
         args = parser.parse_args()
 
         setproctitle(proctitle + '   # gateway main')
-        univention.logging.basicConfig(filename='stdout', univention_debug_level=args.debug)
+
+        univention.logging.basicConfig(filename='stdout', univention_debug_level=args.debug, use_structured_logging=ucr.is_true('directory/manager/rest/debug/structured-logging'))
+        univention.logging.extendLogger('tornado', univention_debug_category='NETWORK')
+        logger = logging.getLogger('tornado')
+        logger.set_ud_level(ucr.get_int('directory/manager/rest/tornado-debug/level', 3))
+        init_request_context_logging(request_context)
 
         tornado.httpclient.AsyncHTTPClient.configure('tornado.curl_httpclient.CurlAsyncHTTPClient')
         tornado.locale.load_gettext_translations('/usr/share/locale', 'univention-directory-manager-rest')
 
         os.umask(0o077)  # FIXME: should probably be changed, this is what UMC sets
 
-        channel = logging.StreamHandler()
-        channel.setFormatter(tornado.log.LogFormatter(fmt='%(color)s%(asctime)s  %(levelname)8s      (%(process)9d) :%(end_color)s%(message)s', datefmt='%d.%m.%y %H:%M:%S'))
-        logger = logging.getLogger('')
-        logger.setLevel(logging.INFO)
-        logger.addHandler(channel)
-
-        init_request_id_logging(request_id_context)
-
         # bind sockets
         socks = []
         if args.port:
@@ -199,7 +192,7 @@ class Gateway(tornado.web.RequestHandler):
             try:
                 child_id = tornado.process.fork_processes(args.processes, 0)
             except RuntimeError as exc:
-                logger.info('Stopped process %s', exc)
+                log.info('Stopped process', error=exc)
                 cls.signal_handler_stop(signal.SIGTERM, None)
             else:
                 cls.start_child(child_id)
@@ -210,15 +203,16 @@ class Gateway(tornado.web.RequestHandler):
     def start_child(cls, child_id):
         setproctitle(proctitle + f'   # gateway proxy {child_id}')
         cls.child_id = child_id
-        logger.info('Started child %s', cls.child_id)
+        log.info('Started child', child=cls.child_id)
         shared_memory.children[cls.child_id] = os.getpid()
         cls.start_server(cls.socks)
 
     @classmethod
     def start_server(cls, socks):
-        app = tornado.web.Application([
-            (r'.*', cls),
-        ], serve_traceback=ucr.is_true('directory/manager/rest/show-tracebacks', True),
+        app = tornado.web.Application(
+            [(r'.*', cls)],
+            serve_traceback=ucr.is_true('directory/manager/rest/show-tracebacks', True),
+            log_function=cls.log_function,
         )
         server = tornado.httpserver.HTTPServer(app)
         server.add_sockets(socks)
@@ -229,6 +223,23 @@ class Gateway(tornado.web.RequestHandler):
             cls.signal_handler_stop(signal.SIGTERM, None)
             raise
 
+    @classmethod
+    def log_function(cls, handler):
+        if handler.get_status() < 400:
+            return  # ignore successfull requests here, they are logged in the other process
+            log_method = logging.getLogger('tornado.access').info
+        elif handler.get_status() < 500:
+            log_method = logging.getLogger('tornado.access').warning
+        else:
+            log_method = logging.getLogger('tornado.access').error
+        request_time = 1000.0 * handler.request.request_time()
+        log_method(
+            "[GATEWAY] %d %s %.2fms",
+            handler.get_status(),
+            handler._request_summary(),
+            request_time,
+        )
+
     @classmethod
     def get_locale(cls, language):
         locale = univention.lib.i18n.Locale(language)
@@ -268,7 +279,7 @@ class Gateway(tornado.web.RequestHandler):
 
     @classmethod
     def signal_handler_reload(cls, sig, frame):
-        ud.debug(ud.MAIN, ud.ALL, 'Reloading service.')
+        log.debug('Reloading service.')
         if cls.child_id is None:
             for process in cls.PROCESSES.values():
                 cls.safe_kill(process.pid, sig)
@@ -286,16 +297,16 @@ class Gateway(tornado.web.RequestHandler):
                 children_pids = list(shared_memory.children.values())
             except Exception:  # multiprocessing failure
                 children_pids = []
-            logger.info('stopping children: %r', children_pids)
+            log.info('stopping children', children=children_pids)
             for pid in children_pids:
                 cls.safe_kill(pid, sig)
-            logger.info('stopping subprocesses: %r', list(cls.PROCESSES.keys()))
+            log.info('stopping subprocesses', subprocesses=list(cls.PROCESSES.keys()))
             for process in cls.PROCESSES.values():
                 cls.safe_kill(process.pid, sig)
 
             shared_memory.shutdown()
         else:
-            logger.info('shutting down')
+            log.info('shutting down')
 
         io_loop = tornado.ioloop.IOLoop.current()
 
@@ -309,6 +320,6 @@ class Gateway(tornado.web.RequestHandler):
         try:
             os.kill(pid, signo)
         except OSError as exc:
-            logger.error('Could not kill(%s) %s: %s', signo, pid, exc)
+            log.error('Could not kill children', signo=signo, pid=pid, error=exc)
         else:
             os.waitpid(pid, os.WNOHANG)
